"Classes for different types of questions"

from typeguard import typechecked

from typing import Optional, List, Union, Sequence
from json import dumps
from options import QuestionOptions

@typechecked
class Question:
    "General question class"

    def __init__(self, label: str, question_type: str, question_text: str, *answers: Union[str, Sequence[str]], options: Optional[QuestionOptions]=None, description: Optional[str]=None):
        self.label = label
        self.question_type = question_type
        self.question_text = question_text
        self.answers = answers
        self.options = options
        self.description = description

    def __str__(self):
        answers = "  - " + "\n  - ".join(self.answers)
        return f"{self.label}:\n{self.question_text} ({self.question_type})\n{answers}"

    def __repr__(self):
        return f"Question({self.label})"

    def json(self):
        "Converts question to SurveyJS-compliant json"
        from structure import SurveyEncoder
        return dumps(self, cls=SurveyEncoder, indent=4)

def radio(self, label: str, question_text: str, *answers: Union[str, List[str]], options: Optional[QuestionOptions]=None, description: Optional[str]=None):
    "Single choice question"
    return Question(label, "radio", options, question_text, answers, options=options, description=description)

def radio(self, label: str, question_text: str, *answers: Union[str, List[str]], options: Optional[QuestionOptions]=None, description: Optional[str]=None):
    "Single choice question"
    return Question(label, "checkbox", options, question_text, answers, options=options, description=description)



from typeguard import typechecked

from typing import List
from question_types import Question
from structure import *

def question(label: str, question_type: str, question_text: str, *answers: Union[str, List[str]], options: Optional[QuestionOptions]=None, description: Optional[str]=None, options: Optional[PageOptions]=None):
    "Wrapper around Question class"
    return Question(label, question_type, question_text, answers, options=options, description=description)

def questionnaire(label, items, answers, question_type="radio", options=None, description=None) -> List[Question]:
    "Convert whole questionnaire to Question objects list"
    q_list = []
    for i in enumerate(items):
        q_list.append(Question(f"{label}_{i[0] + 1}", question_type, i[1], answers, options=options, description=description))
    return q_list

def page(label: str, *questions: Union[Question, List[Question]], title: Optional[str]=None, description: Optional[str]=None, options: Optional[PageOptions]=None):
    "Wrapper around Page class"
    return Page(label, questions, title=title, description=description, options=options)

def survey(*pages: Union[Page, List[Page]], title: Optional[str]=None, description: Optional[str]=None):
    "Create Survey object from pages, create json file"
    s = Survey(pages, title=title, description=description)
    s.create()
    return s



from typeguard import typechecked

from typing import Optional

@typechecked
class QuestionOptions:
    def __init__(self, required: bool=False, answers_order: str="none", inherit_answers: Optional[str]=None, comment: bool=False, comment_text: str="Other", comment_placeholder: str="", visible: bool=True, other: bool=False, other_text: str="Other", other_placeholder: str="", none: bool=False, none_text: str="None", clear_button: bool=False):
        self.required = required
        self.answers_order = answers_order
        self.inherit_answers = inherit_answers
        self.comment = comment
        self.comment_text = comment_text
        self.comment_placeholder = comment_placeholder
        self.visible = visible
        self.other = other
        self.other_text = other_text
        self.other_placeholder = other_placeholder
        self.none = none
        self.none_text = none_text
        self.clear_button = clear_button

class PageOptions:
    def __init__(self, read_only: bool=False, time_limit: Optional[int]=None, visible: bool=True):
        self.read_only = read_only
        self.time_limit = time_limit
        self.visible = visible
        
class SurveyOptions:
    def __init__(self, language: str="en", url_on_complete: Optional[str]=None):
        self.language = language
        self.url_on_complete = url_on_complete



from typeguard import typechecked

from typing import Union, List, Optional
from json import JSONEncoder
from numpy import concatenate, array
from question_types import Question
from options import *

@typechecked
class Page:
    def __init__(self, label: str, *questions: Union[Question, List[Question]], title: Optional[str]=None, description: Optional[str]=None, options: Optional[PageOptions]=None):
        self.label = label
        self.questions = list(concatenate([questions]).flat)
        self.title = title
        self.description = description
        self.options = options

@typechecked
class Survey:
    def __init__(self, *pages: Union[Page, List[Page]], title: Optional[str]=None, description: Optional[str]=None, options: Optional[SurveyOptions]=None):
        self.pages = list(concatenate([pages]).flat)
        self.title = title
        self.description = description
        self.options = options
    
    def json(self):
        from json import dumps
        return dumps(obj=self, cls=SurveyEncoder, indent=2)
    
    def create(self):
        from json import dumps
        json = dumps(obj=self, cls=SurveyEncoder, indent=2)
        f = open("survey.json", "w")
        f.write(json)
        f.close()

class SurveyEncoder(JSONEncoder):
    "Create SurveyJS-compliant json from Question object"

    def default(self, o):
        if isinstance(o, Survey):
            json = {
                "title": o.title,
                "description": o.description,
                "pages": [self.default(p) for p in o.pages]
            }
            
            if o.options:
                opts = o.options.__dict__
                if opts["language"] != "en":
                    json["locale"] = opts["language"]
                if opts["url_on_complete"]:
                    json["navigateToUrl"] = opts["url_on_complete"]
            
        elif isinstance(o, Page):
            json = {
                "name": o.label,
                "elements": [self.default(q) for q in o.questions],
            }
            
            if o.title:
                json["title"] = o.title
            if o.description:
                json["description"] = o.description
            if o.options:
                opts = o.options.__dict__
                if opts["read_only"]:
                    json["readOnly"] = True
                if opts["time_limit"]:
                    json["maxTimeToFinish"] = opts["time_limit"]
                if not opts["visible"]:
                    json["visible"] = False
       
        else:
            # SurveyJS types dictionary
            surveyjs_types = {"radio": "radiogroup", "checkbox": "checkbox"}
            
            json = {
                "name": o.label,
                "type": surveyjs_types[o.question_type],
                "title": o.question_text,
                "choices": list(concatenate([o.answers]).flat)
            }
            
            if o.description:
                json["description"] = o.description
                
            if o.options:
                opts = o.options.__dict__
                if not opts["visible"]:
                    json["visible"] = False
                if opts["required"]:
                    json["isRequired"] = True
                if opts["answers_order"] != "none":
                    json["choicesOrder"] = opts["answers_order"]
                if opts["inherit_answers"]:
                    json["choicesFromQuestion"] = opts["inherit_answers"]
                if opts["comment"]:
                    json["showCommentArea"] = True
                    if opts["comment_text"]:
                        json["commentText"] = opts["comment_text"]
                    if opts["comment_placeholder"]:
                        json["commentPlaceholder"] = opts["comment_placeholder"]
                if opts["other"]:
                    json["showOtherItem"] = True
                    if opts["other_text"]:
                        json["otherText"] = opts["other_text"]
                    if opts["other_placeholder"]:
                        json["otherPlaceHolder"] = opts["other_placeholder"]
                if opts["none"]:
                    json["showNoneItem"] = True
                    if opts["none_text"]:
                        json["noneText"] = opts["none_text"]
                if opts["clear_button"]:
                    json["showClearButton"] = True
            
        return json



from setuptools import setup, find_packages

VERSION = "0.0.3"
DESCRIPTION = "Surveys and experiments tool"
LONG_DESCRIPTION = "Free and open source surveys and experiments tool for social sciences"

setup(
    name="VelesResearch",
    version = VERSION,
    author = "Jakub Jędrusiak",
    author_email = "<kuba23031999@gmail.com>",
    description = DESCRIPTION,
    long_description = LONG_DESCRIPTION,
    packages = find_packages(),
    keywords = ['python', 'surveys', 'experiments', 'social sciences'],
    requires=["json", "numpy", "typeguard", "typing"]
)
